SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

IF EXISTS(SELECT *
		  FROM sys.objects
		  WHERE object_id = OBJECT_ID(N'Pron_IngH_Reserva'))
	BEGIN
		DROP PROCEDURE Pron_IngH_Reserva
	END
GO


CREATE PROCEDURE Pron_IngH_Reserva
	   @dia_inic DATETIME, 
	   @dia_fin DATETIME, 
	   @dia_reserv DATETIME, 
	   @mone INT, 
	   @tasa_ext NUMERIC(13, 8),
	   @id_oper INT
AS

/* Fecha: 01 Mayo 2013
   Calcula ingreso de Hospedaje en RESERVAS,
   dentro un rango de fechas a base de la,
   fecha de reservacion hacia el pasado 
*/

BEGIN

	DECLARE @Ing_Local NUMERIC(15, 2), 
			@mese VARCHAR(10), 
			@ciclo DATETIME, 
			@ing_reserva NUMERIC(15, 2)

	CREATE TABLE #Tab_PRH
	(
		id_rese INT, 
		id_cont INT, 
		id_estado INT, 
		id_mone INT, 
		tasa NUMERIC(13, 8), 
		fecha_ent DATETIME, 
		fecha_sal DATETIME, 
		fecha_reserv DATETIME, 
		cant_habi INT, 
		precio_hab MONEY, 
		nombre VARCHAR(40), 
		nomb_reserv VARCHAR(40), 
		nro_reserva INT, 
		fecha_anulada DATETIME, 
		id_lres INT, 
		id_ralo INT, 
		id_agen INT, 
		tasa_fija NUMERIC(13, 8), 
		cod_mone CHAR(3)
	)
	
	/* CREA TABLA DE POSIBLES DATOS Pasado--Futuro */

	INSERT INTO #Tab_PRH
	SELECT rese.id_rese, 
		   rese.id_cont, 
		   id_estado, 
		   cont.id_mone, 
		   sicla.tasa, 
		   fecha_ent, 
		   fecha_sal, 
		   fecha_reserv, 
		   cant_habi, 
		   precio_hab, 
		   nombre, 
		   nomb_reserv, 
		   nro_reserva, 
		   fecha_anulada, 
		   id_lres, 
		   id_ralo, 
		   id_agen, 
		   tasa_fija, 
		   cod_mone
	FROM       hoterese rese
	INNER JOIN Hotelres lres ON rese.id_rese = lres.id_rese
	INNER JOIN Hotecont cont ON rese.id_cont = cont.id_cont
	INNER JOIN Siclamon sicla ON cont.id_mone = sicla.id_mone
	WHERE (@id_oper = 0 OR rese.id_oper = @id_oper)
	  AND rese.id_estado IN (2, 3, 5)
	  AND CAST(rese.fecha_reserv AS DATE) <= CAST(@dia_reserv AS DATE)
	
	--UNION
	INSERT INTO #Tab_PRH
	SELECT rese.id_rese, 
		   rese.id_cont, 
		   id_estado, 
		   cont.id_mone, 
		   sicla.tasa, 
		   fecha_ent, 
		   fecha_sal, 
		   fecha_reserv, 
		   cant_habi, 
		   precio_hab, 
		   nombre, 
		   nomb_reserv, 
		   nro_reserva, 
		   fecha_anulada, 
		   id_lres, 
		   id_ralo, 
		   id_agen, 
		   tasa_fija, 
		   cod_mone
	FROM       HOTEHRES rese
	INNER JOIN HOTEHLRE lres ON rese.id_rese = lres.id_rese
	INNER JOIN Hotecont cont ON rese.id_cont = cont.id_cont
	INNER JOIN Siclamon sicla ON cont.id_mone = sicla.id_mone
	WHERE (@id_oper = 0 OR rese.id_oper = @id_oper)
	  AND rese.id_estado IN (2, 3, 5)
	  AND CAST(rese.fecha_reserv AS DATE) <= CAST(@dia_reserv AS DATE)
	ORDER BY fecha_reserv

	DELETE FROM #Tab_PRH
	WHERE id_estado = 2 
	  AND id_rese  IN  (SELECT ISNULL(id_rese, 0) 
	                    FROM   Hoteface
	                    WHERE  CAST(fecha_ent AS DATE) <= @dia_reserv)

	/* CREA CONSULTA DATOS DE INGRESO DE HOSPEDAJE POR MES */ 

	-- Mejor que quemar el impuesto

	DECLARE @ImpuHospedaje NUMERIC(18, 2) = ISNULL((SELECT TOP 1 SUM(im.porc_impu)impu_hosp
													FROM       HOTETHAB th
													INNER JOIN HOTECON0 c0 ON th.id_con0 = c0.id_con0
													INNER JOIN HOTEPRE0 p0 ON c0.id_con0 = p0.id_con0
													INNER JOIN HOTEIMPU ip ON p0.id_pre0 = ip.id_pre0
													INNER JOIN SICLAIMP im ON ip.id_impu = im.id_impu
													INNER JOIN SICLAMON mo ON p0.id_mone = mo.id_mone
													WHERE mo.tasa = 1 --AND ip.incluido = 1
													GROUP BY th.id_thab), 0) / 100 + 1

	SET @mese = LEFT(YEAR(@dia_inic), 4) + RIGHT('0' + LEFT(MONTH(@dia_inic), 2), 2)

	WHILE @mese <= LEFT(YEAR(@dia_fin), 4) + RIGHT('0' + LEFT(MONTH(@dia_fin), 2), 2)
		BEGIN 
 
			SELECT @ciclo = DATEADD(s,-1,DATEADD(mm, DATEDIFF(m,0,@dia_inic)+1,0)) -- Obtener ltimo da del mes en base a la fecha dada.

			SET @ing_reserva = 0
			WHILE @dia_inic <= @ciclo
				BEGIN
				
					SELECT @ing_local = SUM(CONVERT(NUMERIC(15, 2), precio_hab / @ImpuHospedaje) * cant_habi * CASE WHEN id_mone != @mone THEN @tasa_ext / tasa
																													ELSE 1
																											        END)
					FROM #Tab_PRH
					WHERE DATEDIFF(DAY, @dia_reserv, fecha_reserv) <= 0
					  AND DATEDIFF(day, fecha_ent, @dia_inic) >= 0
					  AND DATEDIFF(day, @dia_inic, fecha_sal) > 0
					  
					INSERT DEBGDAIPG(fecha, Ing_Reserva, ing_reserva_Reg, ing_reserva_Otr, ing_Ro_Al, ing_Ro_Pen, ing_ROAgen)
					VALUES(CAST(@dia_inic AS DATE), COALESCE(@Ing_Local, 0), 0, 0, 0, 0, 0)

					SET @dia_inic = @dia_inic + 1
					SET @ing_reserva = @ing_reserva + COALESCE(@Ing_Local, 0)
					
					SET @Ing_Local = 0
				END   	
		
			/* CREA TABLA CON DATOS DE INGRESO DE HOSPEDAJE POR MES */

			INSERT INTO HOTEDAIPG
			VALUES(MONTH(@ciclo), 
				   YEAR(@ciclo), 
				   @ing_reserva, 
				   0, 
				   0, 
				   0, 
				   0, 
				   0)

			SET @mese = LEFT(YEAR(@dia_inic), 4) + RIGHT('0' + LEFT(MONTH(@dia_inic), 2), 2)

		END
END